import { Dialog, DialogFooter, PrimaryButton, DefaultButton, CommandBar, ShimmeredDetailsList, SelectionMode, DetailsListLayoutMode, ConstrainMode, Selection, Icon, Stack } from "@fluentui/react";
import { useState, useEffect, useRef, useCallback } from "react";
import { FluentDialog } from "./FluentDialog";
import React from "react";
import { useStateRef, formListHelper, ValidationResult, lang, id } from "@wiberg/formbuilder";
import { Searcher } from "../..";
const FluentList = (props) => {
    const editorFormRef = useRef();
    const [items, setItems] = useState(props.items);
    const [filteredItems, setFilteredItems] = useState(props.filteredItems);
    const [showEditor, setShowEditor] = useState(false);
    const [columns, setColumns, columnRef] = useStateRef();
    const [showValidationOverrideConfirm, setShowValidationOverrideConfirm] = useState(false);
    const [enableShimmer, setEnableShimmer] = useState(false);
    const [newItemMode, setNewItemMode] = useState(false);
    const [menuItems, setMenuItems] = useState(undefined);
    const [validationFailed, setValidationFailed] = useState(false);
    const [selection] = useState(new Selection({
        selectionMode: props.listProps.config.multiSelect ? SelectionMode.multiple : SelectionMode.single,
        getKey: (item, index) => props.listProps.config.itemIdentifier(item),
        onSelectionChanged: () => {
            props.updateSelectedItems(selection.getSelection() ?? []);
        }
    }));
    useEffect(() => {
        let cols = buildColumns();
        setColumns(cols);
    }, [props.columns]);
    useEffect(() => {
        setEnableShimmer(props.shimmer);
    }, [props.shimmer]);
    useEffect(() => {
        if (props.items?.length)
            setEnableShimmer(false);
        setItems(props.items);
    }, [props.items]);
    useEffect(() => {
        setFilteredItems(props.filteredItems);
    }, [props.filteredItems]);
    const buildMenuItem = useCallback((item) => {
        return {
            key: item.title,
            text: item.title,
            onClick: () => item.action(props.selectedItems),
            disabled: props.readOnly || formListHelper.isMenuItemDisabled(item.selectionMode, props.selectedItems?.length),
            iconProps: { iconName: item.iconName },
            subMenuProps: item.subMenuItems == null
                ? undefined
                : { items: item.subMenuItems.map(buildMenuItem) }
        };
    }, [items, props.selectedItems, props.readOnly]);
    useEffect(() => {
        if (props.listProps.menuConfig?.actions == null)
            return;
        let actions = props.listProps.menuConfig?.actions(createItem, editItem, deleteItems);
        if (!actions?.length)
            return; // is it needed to set to empty array as well? (ewi)
        // parse to commandbarProps
        let commandBarItems = actions.map(buildMenuItem);
        setMenuItems(commandBarItems);
    }, [props.listProps.menuConfig, props.selectedItems]);
    const buildColumns = () => {
        return props.columns.map(_ => {
            let col = _;
            col.onColumnClick = () => props.sortColumn(_);
            const resizable = props.listProps.columnConfig?.resizableColumns;
            if (resizable === true || (resizable?.findIndex(_ => _ == col.fieldName) ?? -1) >= 0)
                col.isResizable = true;
            const widthConfig = props.listProps.columnConfig?.columnWidths;
            if (widthConfig != null) {
                // do not override width from item schema
                if (typeof widthConfig === 'number' && col.minWidth == null) {
                    col.minWidth = widthConfig;
                }
                else if (Array.isArray(widthConfig)) {
                    const config = widthConfig?.find(_ => _.column == col.fieldName);
                    if (config?.width != null)
                        col.minWidth = config.width;
                }
            }
            return col;
        });
    };
    // TODO: move crud operations out to FormList (ewi)
    //       with alot of other stuff as well
    const createItem = (pre) => {
        let item = {};
        if (pre) {
            let result = pre(item);
            if (result === false)
                return;
        }
        selection.setAllSelected(false);
        props.updateSelectedItems([item]);
        setNewItemMode(true);
        setShowEditor(true);
    };
    const editItem = (pre) => {
        let sItem = { ...props.selectedItems[0] };
        if (pre) {
            let result = pre(sItem);
            if (result === false)
                return;
        }
        props.updateSelectedItems([sItem]);
        // setTimeout(() => {
        setNewItemMode(false);
        setShowEditor(true);
        // }, 100); // TODO: why was this needed ? (ewi) (200 before)
    };
    const deleteItems = (pre) => {
        const success = props.deleteItems(pre);
        if (!success)
            return;
        selection.setAllSelected(false);
    };
    const validateForm = async () => {
        let validated = await editorFormRef.current?.validateItem();
        return validated;
    };
    const saveForm = async () => {
        let validated = await validateForm();
        if (validated == ValidationResult.FailedDontBlock) {
            setShowValidationOverrideConfirm(true);
        }
        else if (validated == ValidationResult.Failed) {
            setValidationFailed(true);
        }
        else if (validated == ValidationResult.Success) {
            props.onItemChange(editorFormRef.current?.getItem());
            setValidationFailed(false);
            setShowEditor(false);
        }
    };
    const dismissForm = async () => {
        if (newItemMode)
            props.updateSelectedItems([]);
        setNewItemMode(false);
        setShowEditor(false);
    };
    const getItem = () => {
        return editorFormRef.current?.getItem();
    };
    const onRenderFooterContent = useCallback(() => {
        if (props.listProps?.editorConfig?.customFooter != undefined)
            return props.listProps.editorConfig.customFooter(saveForm, dismissForm, validateForm, getItem);
        else
            return (React.createElement(Stack, { horizontal: true },
                React.createElement(PrimaryButton, { text: newItemMode ? lang.texts.areas.common.create : lang.texts.areas.common.save, styles: { root: { marginRight: 8 } }, onClick: saveForm }),
                React.createElement(DefaultButton, { text: lang.texts.areas.common.cancel, onClick: dismissForm }),
                validationFailed && React.createElement("div", { style: { paddingLeft: "10px", paddingTop: "2px" } },
                    React.createElement(Icon, { iconName: "Error", title: "Validation errors", styles: { root: { color: "red", fontSize: "20px", cursor: "help" } } }))));
    }, [validationFailed]);
    const handleColumnReorder = (draggedIndex, targetIndex) => {
        if (columns == null)
            return;
        const draggedItems = columns[draggedIndex];
        const newColumns = [...columns];
        // insert before the dropped item
        newColumns.splice(draggedIndex, 1);
        newColumns.splice(targetIndex, 0, draggedItems);
        setColumns(newColumns);
    };
    return React.createElement(React.Fragment, null,
        (showEditor && props.selectedItems?.length)
            && React.createElement(FluentDialog, { item: props.selectedItems[0], editorConfig: props.listProps.editorConfig, formListProps: props, keyPrefix: props.keyPrefix ?? "", newItemMode: newItemMode, onRenderFooterContent: onRenderFooterContent, show: (show) => {
                    setValidationFailed(false);
                    setShowEditor(show);
                }, ref: editorFormRef }),
        React.createElement("div", { className: "formbuilder-listcontainer-toolbar" },
            props.listProps.searchConfig?.searchEnabled &&
                React.createElement(Searcher, { items: items, onSearch: (filteredItems, searchText) => {
                        props.updateFilteredItems(filteredItems);
                        selection.setAllSelected(false);
                        props.updateSelectedItems([]);
                    }, className: "tp-searcher", delay: 600, searchPlaceHolder: props.listProps.searchConfig.searchPlaceHolder, enableTextHighlight: true, searchFields: props.listProps.searchConfig.searchableFields, highLightElements: () => Array.from(document.querySelectorAll('.ms-List .ms-DetailsRow-cell:not(.ms-DetailsRow-cellCheck)'))
                        .filter(_ => {
                        if (props.listProps.searchConfig?.searchableFields == null)
                            return [];
                        return props.listProps.searchConfig.searchableFields.findIndex(sf => sf == _.attributes["data-automation-key"].value) >= 0;
                    }), filterOnItemsChange: true }),
            menuItems !== undefined &&
                React.createElement(CommandBar, { items: menuItems, styles: { root: { padding: 0 } } })),
        React.createElement("div", { className: "formbuilder-listcontainer-list" },
            React.createElement(ShimmeredDetailsList, { columns: columns, items: filteredItems, 
                // selectionPreservedOnEmptyClick
                selection: selection, enableShimmer: enableShimmer, shimmerLines: props.listProps?.shimmerConfig?.shimmerLines, layoutMode: DetailsListLayoutMode.fixedColumns, constrainMode: ConstrainMode.horizontalConstrained, setKey: 'itemId', getKey: (item) => `item-${props.listProps.config.itemIdentifier(item) ?? id.make()}`, onItemInvoked: props.listProps.config.disableItemInvoke ? undefined :
                    (item, index, ev) => {
                        if (item == null) {
                            props.updateSelectedItems([]);
                        }
                        else {
                            props.updateSelectedItems([item]);
                        }
                        editItem();
                    }, columnReorderOptions: columns?.length ? { handleColumnReorder } : undefined, useReducedRowRenderer: true })),
        (!enableShimmer && !!!items?.length) && (props.listProps?.shimmerConfig?.noItemsElement ?? null),
        React.createElement(Dialog, { hidden: !showValidationOverrideConfirm, title: lang.texts.areas.list.validationRequirementsNotMet.title, onDismiss: () => setShowValidationOverrideConfirm(false), dialogContentProps: {
                subText: lang.texts.areas.list.validationRequirementsNotMet.desctiption
            }, modalProps: {
                isBlocking: true
            } },
            React.createElement(DialogFooter, null,
                React.createElement(PrimaryButton, { text: lang.texts.areas.list.validationRequirementsNotMet.ok, onClick: () => {
                        setShowValidationOverrideConfirm(false);
                        setShowEditor(false);
                        props.onItemChange(editorFormRef.current?.getItem());
                    } }),
                React.createElement(DefaultButton, { text: lang.texts.areas.list.validationRequirementsNotMet.cancel, onClick: () => setShowValidationOverrideConfirm(false) }))));
};
export default FluentList;
//# sourceMappingURL=FluentList.js.map